@@ -1,14 +1,38 @@
-//! # Apollo Studio Extension for Performance Tracing for async_graphql crates
+//! # Apollo Extensions for async_graphql
+//!  <div align="center">
+//!  <!-- CI -->
+//!  <img src="https://github.com/Miaxos/async_graphql_apollo_studio_extension/actions/workflows/ci.yml/badge.svg" />
+//!  <!-- Crates version -->
+//!  <a href="https://crates.io/crates/async-graphql-extension-apollo-tracing">
+//!    <img src="https://img.shields.io/crates/v/async-graphql-extension-apollo-tracing.svg?style=flat-square"
+//!    alt="Crates.io version" />
+//!  </a>
+//!  <!-- Downloads -->
+//!  <a href="https://crates.io/crates/async-graphql-extension-apollo-tracing">
+//!    <img src="https://img.shields.io/crates/d/async-graphql-extension-apollo-tracing.svg?style=flat-square"
+//!      alt="Download" />
+//!  </a>
+//! </div>
+//!
+//! ## Features
+//!
+//! * Fully support traces & errors
+//! * Batched traces transfer
+//! * Client segmentation
+//! * Tracing
+//! * Schema register protocol implemented
+//!
+//! ## Crate Features
+//!
+//! * `compression` - To enable GZIP Compression when sending traces to Apollo Studio.
+mod compression;
 mod packages;
 mod proto;
 pub mod register;
+mod runtime;
 
 #[macro_use]
 extern crate tracing;
-/*
- * TODO:
- *   - Gzip compression (libflate)
- */
 use packages::uname::Uname;
 use std::collections::HashMap;
 use std::sync::Arc;
@@ -30,15 +54,16 @@ use proto::{
     Report, ReportHeader, Trace, Trace_Details, Trace_Error, Trace_HTTP, Trace_HTTP_Method,
     Trace_Location, Trace_Node, Trace_Node_oneof_id, TracesAndStats,
 };
+use runtime::{channel, Runtime, RwLock, Sender};
 use std::convert::TryInto;
-use tokio::sync::mpsc::{channel, Sender};
-use tokio::sync::RwLock;
 
-/// Apollo tracing extension for performance tracing
-/// https://www.apollographql.com/docs/studio/setup-analytics/#adding-support-to-a-third-party-server-advanced
+/// Apollo Tracing Extension to send traces to Apollo Studio
+/// The extension to include to your `async_graphql` instance to connect with Apollo Studio.
 ///
-/// Apollo Tracing works by creating Trace from GraphQL calls, which contains extra data about the
-/// request being processed. These traces are then batched sent to the Apollo Studio server.
+/// <https://www.apollographql.com/docs/studio/setup-analytics/#adding-support-to-a-third-party-server-advanced>
+///
+/// Apollo Tracing works by creating traces from GraphQL calls, which contains extra data about the
+/// request being processed. These traces are then batched sent to Apollo Studio.
 ///
 /// The extension will start a separate function on a separate thread which will aggregate traces
 /// and batch send them.
@@ -54,6 +79,7 @@ const TARGET_LOG: &str = "apollo-studio-extension";
 const VERSION: &str = env!("CARGO_PKG_VERSION");
 const RUNTIME_VERSION: &str = "Rust - No runtime version provided yet";
 
+/// An ENUM describing the various HTTP Methods existing.
 #[derive(Debug, Clone)]
 pub enum HTTPMethod {
     UNKNOWN = 0,
@@ -85,7 +111,30 @@ impl From<HTTPMethod> for Trace_HTTP_Method {
     }
 }
 
-/// This structure must be registered to the Query Data to add context to the apollo metrics.
+/// The structure where you can add additional context for Apollo Studio.
+/// This structure must be added to your query data.
+///
+/// It'll allow you to [segment your
+/// users](https://www.apollographql.com/docs/studio/client-awareness/)
+///
+/// * `userid` - To segment your users, you should fill this field with an internal id describing
+/// your client or event a name. Apollo Studio will be able to aggregate metrics by users.
+/// * `client_name` - You can segment your users by the client they are using to access your
+/// GraphQL API, it's really usefull when you have mobile and web users for instance. Usually we
+/// add a header `apollographql-client-name` to store this data. Apollo Studio will allow you to
+/// aggregate your metrics by Client Name.
+/// * `client_version` - You can segment your users by the client but it's usefull to also have the
+/// version your clients are using, especially when you are serving your API for mobile users,
+/// it'll allow you to follow metrics depending on which version your users are. Usually we add a
+/// header `apollographql-client-version` to store this data.
+/// * `path` - It's the HTTP path to your GraphQL API, it may be usefull for you but generally it's
+/// just `/graphql`.
+/// * `host` - It's the HTTP host to your GraphQL API.
+/// * `method` - The HTTP Method.
+/// * `secure` - If you have SSL.
+/// * `protocol` - The http protocol, example: HTTP/1, HTTP/1.1, HTTP/2.
+/// * `status_code` - The status code return by your GraphQL API. It's a little weird to have to put it
+/// before executing the graphql function, it'll be changed later but usually it's just a 200.
 #[derive(Debug, Clone)]
 pub struct ApolloTracingDataExt {
     pub userid: Option<String>,
@@ -119,12 +168,13 @@ impl ApolloTracing {
     /// We initialize the ApolloTracing Extension by starting our aggregator async function which
     /// will receive every traces and send them to the Apollo Studio Ingress for processing
     ///
-    /// autorization_token - Token to send metrics to apollo studio.
-    /// hostname - Hostname like brevz-graphql-1.io
-    /// graph_ref - <ref>@<variant> Graph reference with variant
-    /// release_name - Your release version or release name from Git for example
-    /// batch_target - The number of traces to batch, it depends on your traffic
-    pub fn new<'a>(
+    /// * autorization_token - Token to send metrics to apollo studio.
+    /// * hostname - Hostname like yourdomain-graphql-1.io
+    /// * graph_ref - <ref>@<variant> Graph reference with variant
+    /// * release_name - Your release version or release name from Git for example
+    /// * batch_target - The number of traces to batch, it depends on your traffic, if you have
+    /// 60 request per minutes, set it to 20.
+    pub fn new(
         authorization_token: String,
         hostname: String,
         graph_ref: String,
@@ -135,7 +185,7 @@ impl ApolloTracing {
             uname: Uname::new()
                 .ok()
                 .map(|x| x.to_string())
-                .unwrap_or("No uname provided".to_string()),
+                .unwrap_or_else(|| "No uname provided".to_string()),
             hostname,
             graph_ref,
             service_version: release_name,
@@ -149,11 +199,16 @@ impl ApolloTracing {
 
         let header_tokio = Arc::clone(&header);
 
-        tokio::spawn(async move {
+        Runtime::locate().spawn(async move {
             let mut hashmap: HashMap<String, TracesAndStats> =
                 HashMap::with_capacity(batch_target + 1);
             let mut count = 0;
-            while let Some((name, trace)) = receiver.recv().await {
+            while let Some((name, trace)) = match Runtime::locate() {
+                #[cfg(feature = "tokio-comp")]
+                Runtime::Tokio => receiver.recv().await,
+                #[cfg(feature = "async-std-comp")]
+                Runtime::AsyncStd => receiver.recv().await.ok(),
+            } {
                 trace!(target: TARGET_LOG, message = "Trace registered", trace = ?trace, name = ?name);
 
                 // We bufferize traces and create a Full Report every X
@@ -164,13 +219,13 @@ impl ApolloTracing {
                     }
                     None => {
                         let mut trace_and_stats = TracesAndStats::new();
-                        &trace_and_stats.mut_trace().push(trace);
+                        trace_and_stats.mut_trace().push(trace);
 
                         hashmap.insert(name, trace_and_stats);
                     }
                 }
 
-                count = count + 1;
+                count += 1;
 
                 if count > batch_target {
                     use tracing::{field, field::debug, span, Level};
@@ -203,13 +258,25 @@ impl ApolloTracing {
                         }
                     };
 
-                    let result = client
+                    let mut client = client
                         .post(REPORTING_URL)
-                        .body(msg)
                         .header("content-type", "application/protobuf")
-                        .header("X-Api-Key", &authorization_token)
-                        .send()
-                        .await;
+                        .header("accept", "application/json")
+                        .header("X-Api-Key", &authorization_token);
+
+                    if cfg!(feature = "compression") {
+                        client = client.header("content-encoding", "gzip");
+                    };
+
+                    let msg = match compression::compress(msg) {
+                        Ok(result) => result,
+                        Err(e) => {
+                            error!(target: TARGET_LOG, message = "An issue happened while GZIP compression", err = ?e);
+                            continue;
+                        }
+                    };
+
+                    let result = client.body(msg).send().await;
 
                     match result {
                         Ok(data) => {
@@ -313,22 +380,28 @@ impl Extension for ApolloTracingExtension {
             .data::<ApolloTracingDataExt>()
             .ok()
             .cloned()
-            .unwrap_or(ApolloTracingDataExt::default());
+            .unwrap_or_else(ApolloTracingDataExt::default);
         let client_name = tracing_extension
             .client_name
-            .unwrap_or("no client name".to_string());
+            .unwrap_or_else(|| "no client name".to_string());
         let client_version = tracing_extension
             .client_version
-            .unwrap_or("no client version".to_string());
-        let userid = tracing_extension.userid.unwrap_or("anonymous".to_string());
+            .unwrap_or_else(|| "no client version".to_string());
+        let userid = tracing_extension
+            .userid
+            .unwrap_or_else(|| "anonymous".to_string());
 
-        let path = tracing_extension.path.unwrap_or("no path".to_string());
-        let host = tracing_extension.host.unwrap_or("no host".to_string());
+        let path = tracing_extension
+            .path
+            .unwrap_or_else(|| "no path".to_string());
+        let host = tracing_extension
+            .host
+            .unwrap_or_else(|| "no host".to_string());
         let method = tracing_extension.method.unwrap_or(HTTPMethod::UNKNOWN);
         let secure = tracing_extension.secure.unwrap_or(false);
         let protocol = tracing_extension
             .protocol
-            .unwrap_or("no protocol".to_string());
+            .unwrap_or_else(|| "no operation".to_string());
         let status_code = tracing_extension.status_code.unwrap_or(0);
 
         let mut trace = Trace {
@@ -342,15 +415,15 @@ impl Extension for ApolloTracingExtension {
             ..Default::default()
         };
 
-        &trace.set_details(Trace_Details {
+        trace.set_details(Trace_Details {
             operation_name: operation_name
                 .map(|x| x.to_string())
-                .unwrap_or("no operation".to_string()),
+                .unwrap_or_else(|| "no operation".to_string()),
             ..Default::default()
         });
 
         // Should come from Context / Headers
-        &trace.set_http(Trace_HTTP {
+        trace.set_http(Trace_HTTP {
             path,
             host,
             method: Trace_HTTP_Method::from(method),
@@ -360,33 +433,32 @@ impl Extension for ApolloTracingExtension {
             ..Default::default()
         });
 
-        &trace.set_end_time(Timestamp {
+        trace.set_end_time(Timestamp {
             nanos: inner.end_time.timestamp_subsec_nanos().try_into().unwrap(),
-            seconds: inner.end_time.timestamp().try_into().unwrap(),
+            seconds: inner.end_time.timestamp(),
             ..Default::default()
         });
 
-        &trace.set_start_time(Timestamp {
+        trace.set_start_time(Timestamp {
             nanos: inner
                 .start_time
                 .timestamp_subsec_nanos()
                 .try_into()
                 .unwrap(),
-            seconds: inner.start_time.timestamp().try_into().unwrap(),
+            seconds: inner.start_time.timestamp(),
             ..Default::default()
         });
 
         let root_node = self.root_node.read().await;
-        &trace.set_root(root_node.clone());
+        trace.set_root(root_node.clone());
 
         let sender = self.sender.clone();
 
         let operation_name = self.operation_name.read().await.clone();
-        tokio::spawn(async move {
-            match sender.send((operation_name, trace)).await {
-                Err(e) => error!(error = ?e),
-                _ => {}
-            };
+        Runtime::locate().spawn(async move {
+            if let Err(e) = sender.send((operation_name, trace)).await {
+                error!(error = ?e);
+            }
         });
         resp
     }
@@ -439,8 +511,8 @@ impl Extension for ApolloTracingExtension {
             Ok(res) => Ok(res),
             Err(e) => {
                 let mut error = Trace_Error::new();
-                &error.set_message(e.message.clone());
-                &error.set_location(RepeatedField::from_vec(
+                error.set_message(e.message.clone());
+                error.set_location(RepeatedField::from_vec(
                     e.locations
                         .clone()
                         .into_iter()
@@ -455,7 +527,7 @@ impl Extension for ApolloTracingExtension {
                     Ok(content) => content,
                     Err(e) => serde_json::json!({ "error": format!("{:?}", e) }).to_string(),
                 };
-                &error.set_json(json);
+                error.set_json(json);
                 node.write()
                     .await
                     .set_error(RepeatedField::from_vec(vec![error]));
